#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QQuickItem>
#include <QQmlContext>
#include <QEvent>

MainWindow::MainWindow() :
    QMainWindow(),
    _ui(new Ui::MainWindow)
{
    _ui->setupUi(this);

    QObject::connect(_ui->exit_btn, &QPushButton::clicked, qApp, &QApplication::quit);

    //setup qml map
    _qml_view = new QQuickView();
    QWidget* container = QWidget::createWindowContainer(_qml_view, this);
    container->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    _ui->qml_canvas_layout->addWidget(container);
    _qml_view->setSource(QUrl::fromLocalFile(":/map"));

    //test get/set qml object properties
    {
        auto map_view = _qml_view->rootObject()->findChild<QQuickItem*>("mapView");
        auto coordinates = map_view->property("center").value<QGeoCoordinate>();
        coordinates.setLatitude(coordinates.latitude() + 0.02);
        map_view->setProperty("center", QVariant::fromValue<QGeoCoordinate>(coordinates));
    }

    //add filter for mouse coordinates
    _qml_view->installEventFilter(this);

    //connect QML object and C++ object - message receiver
    _qml_receiver = new QMLReceiver();
    _qml_view->rootContext()->setContextProperty("qmlReceiver", _qml_receiver);
    QObject::connect(_qml_receiver, &QMLReceiver::centerChangedSignal, this, &MainWindow::onCenterChanged);

    //connect QML object and C++ object - map items model
    _model = new MarkerModel();
    _qml_view->rootContext()->setContextProperty("markerModel", _model);
    QObject::connect(_qml_receiver, &QMLReceiver::markerClickedSignal, this, &MainWindow::onMarkerClicked);

    //add several markers
    _model->addMarker("1", "Car Red",  59.9569f, 30.3135f, "http://maps.gstatic.com/mapfiles/ridefinder-images/mm_20_red.png");
    _model->addMarker("2", "Car Blue",  59.9566f, 30.32015f, "http://maps.gstatic.com/mapfiles/ridefinder-images/mm_20_blue.png");
}

MainWindow::~MainWindow()
{
    delete _ui;
    delete _qml_receiver;
    delete _model;
}

bool MainWindow::eventFilter(QObject* obj, QEvent* event)
{
    if(obj == _qml_view &&
       event->type() == QEvent::MouseMove)
    {
        auto coord = getMouseCoordinates();
        QString coord_string = QString("Mouse position: %1;%2").arg(std::get<0>(coord)).arg(std::get<1>(coord));
        setInformationText(MainWindow::TextType::MouseCoord, coord_string);
    }
    return QMainWindow::eventFilter(obj, event);
}

void MainWindow::setInformationText(MainWindow::TextType type, const QString& text)
{
    auto plain_text = _ui->information->toPlainText();
    if(plain_text.isEmpty())
        plain_text = text;
    else
    {
        auto strings = plain_text.split("\n");
        switch(type)
        {
            case MainWindow::TextType::MouseCoord:
                strings[0] = text;
            break;
            case MainWindow::TextType::CenterCoord:
                if(strings.size() >= 2)
                    strings[1] = text;
                else
                    strings.append(text);
            break;
        case MainWindow::TextType::MarkerInfo:
            if(strings.size() >= 3)
                strings[2] = text;
            else
                strings.append(text);
            break;
        }
        plain_text = strings.join("\n");
    }
    _ui->information->setPlainText(plain_text);
}

std::tuple<float, float> MainWindow::getMouseCoordinates()
{
    auto map_view = _qml_view->rootObject()->findChild<QQuickItem*>("mapView");
    QVariant result;
    bool invoke_result = QMetaObject::invokeMethod(map_view, "getMousePosition", Qt::DirectConnection, Q_RETURN_ARG(QVariant, result));
    if(!invoke_result)
        std::make_tuple(0.f, 0.f);
    QGeoCoordinate coordinates = result.value<QGeoCoordinate>();
    return std::make_tuple(coordinates.latitude(), coordinates.longitude());
}

void MainWindow::onCenterChanged(float lat, float lon)
{
    QString map_string = QString("Map center position: %1;%2").arg(lat).arg(lon);
    setInformationText(MainWindow::TextType::CenterCoord, map_string);
}

void MainWindow::onMarkerClicked(const QString&, const QString& name, float lat, float lon)
{
    QString marker_string = QString("'%1' clicked: %2;%3").arg(name).arg(lat).arg(lon);
    setInformationText(MainWindow::TextType::MarkerInfo, marker_string);
}
