#include <QGeoCoordinate>

#include "markermodel.h"

void MarkerModel::addMarker(const QString& identifier, const QString& name, float lat, float lon, const QString& icon)
{
    beginInsertRows(QModelIndex(), rowCount(), rowCount());
    _markers.append(Marker{ identifier, name, QGeoCoordinate(lat, lon), icon });
    endInsertRows();
}

std::tuple<float, float> MarkerModel::getMarkerLatLon(const QString& identifier) const
{
    int row = indexOf(identifier);
    if (row == -1)
        return std::make_tuple(0.f, 0.f);
    return std::make_tuple(_markers[row]._position.latitude(), _markers[row]._position.longitude());
}

void MarkerModel::setMarkerLatLon(const QString& identifier, float lat, float lon)
{
    int row = indexOf(identifier);
    if(row == -1)
        return;
    _markers[row]._position = QGeoCoordinate(lat, lon);
}

void MarkerModel::setMarkerName(const QString& identifier, const QString& name, bool redraw)
{
    int row = indexOf(identifier);
    if(row == -1)
        return;
    _markers[row]._name = name;
    if(redraw)
        emit dataChanged(index(row, 0), index(row, 0));
}

void MarkerModel::setMarkerIcon(const QString& identifier, const QString& icon, bool redraw)
{
    int row = indexOf(identifier);
    if(row == -1)
        return;
    _markers[row]._icon = icon;
    if(redraw)
        emit dataChanged(index(row, 0), index(row, 0));
}

void MarkerModel::delMarker(const QString& identifier)
{
    int row = indexOf(identifier);
    if(row == -1)
        return;
    beginRemoveRows(QModelIndex(), row, row);
    _markers.removeAt(row);
    endRemoveRows();
}

void MarkerModel::clear()
{
    _markers.clear();
    emit layoutChanged();
}

void MarkerModel::redrawCurrent(const QString& identifier)
{
    int row = indexOf(identifier);
    if(row != -1)
        emit dataChanged(index(row, 0), index(row, 0));
}

void MarkerModel::redrawAll()
{
    for(int row = 0; row < _markers.size(); ++row)
        emit dataChanged(index(row, 0), index(row, 0));
}

int MarkerModel::rowCount(const QModelIndex& parent) const
{
    Q_UNUSED(parent)
    return _markers.count();
}

QVariant MarkerModel::data(const QModelIndex& index, int role) const
{
    if(index.row() < 0 ||
       index.row() >= _markers.count())
        return QVariant();
    if (role == MarkerModel::positionRole)
        return QVariant::fromValue(_markers[index.row()]._position);
    else if (role == MarkerModel::identifierRole)
        return QVariant::fromValue(_markers[index.row()]._identifier);
    else if (role == MarkerModel::nameRole)
        return QVariant::fromValue(_markers[index.row()]._name);
    else if(role == MarkerModel::iconRole)
        return QVariant::fromValue(_markers[index.row()]._icon);
    return QVariant();
}

QHash<int, QByteArray> MarkerModel::roleNames() const
{
    QHash<int, QByteArray> roles;
    roles[positionRole] = "positionValue";
    roles[identifierRole] = "identifierValue";
    roles[nameRole] = "nameValue";
    roles[iconRole] = "iconValue";
    return roles;
}


int MarkerModel::indexOf(const QString& identifier) const
{
    for(int i = 0; i < _markers.size(); ++i)
        if(_markers[i]._identifier == identifier)
            return i;
    return -1;
}
